@@ -142,7 +142,7 @@ def checkAndRemoveLeftovers(leftovers, start, end, ret, tokenType):
 
 def figureOutRegex(stopwords, size=2):
     punc = [x for x in string.punctuation]
-    regex = '[\s' + "|\\".join(punc)+ ']{'+ str(size)+',}|\n'
+    regex = '[\s' + "|\\".join(punc)+ ']{'+ str(size)+',}|\n|[\s' + "|\\".join(punc)+ ']+$'
     stops = "|".join(["(\s|^)"+stop+"(\s|$)" for stop in stopwords])
     if len(stops) != 0:
         regex = stops+'|'+regex
@@ -204,3 +204,48 @@ def wildgram(text, stopwords=STOPWORDS, topicwords=TOPICWORDS, include1gram=True
         ret.append(app)
 
     return ret
+
+# rules in the form [{"topic": topic, "spans":  [span(s)], "spanType": x (eg. tokenType, token, etc.)}]
+class WildRules:
+    def __init__(self, rules):
+        self.rules = rules
+    def apply(self, span):
+        tokens = wildgram(span, includeParent = False)
+        if len(tokens) == 0:
+            return []
+        for i in range(len(tokens)):
+            tokens[i]["topic"] = "unknown"
+            for rule in self.rules:
+                if replaceMeaninglessPunc(tokens[i][rule["spanType"]]) in rule["spans"]:
+                    tokens[i]["topic"] = rule["topic"]
+        newtoks = []
+        prevType = tokens[0]["topic"]
+        prevToks = [tokens[0]]
+        for i in range(1,len(tokens)):
+            if len(prevToks) > 0 and tokens[i]["startIndex"] < prevToks[-1]["endIndex"]:
+                continue
+            if tokens[i]["tokenType"] == "noise" and tokens[i]["topic"] == "unknown" and len(tokens[i]["token"]) >= 2:
+                newtoks, prevType, prevToks = resetNewToks(newtoks, prevType, prevToks, span)
+                continue
+            if tokens[i]["tokenType"] == "noise" and tokens[i]["topic"] == "unknown":
+                prevToks.append(tokens[i])
+                continue
+            if prevType != tokens[i]["topic"]:
+                newtoks, prevType, prevToks = resetNewToks(newtoks, prevType, prevToks, span)
+            prevType = tokens[i]["topic"]
+            prevToks.append(tokens[i])
+
+        newtoks, prevType, prevToks = resetNewToks(newtoks, prevType, prevToks, span)
+        return newtoks
+
+def resetNewToks(newtoks, prevType, prevToks, span):
+    if len(prevToks) == 0:
+        return newtoks, "", []
+    if prevToks[-1]["tokenType"] == "noise" and prevToks[-1]["topic"] == "unknown":
+        return resetNewToks(newtoks, prevType, prevToks[:-1], span)
+    snippet = replaceMeaninglessPunc(span[prevToks[0]["startIndex"]:prevToks[-1]["endIndex"]])
+    newtoks.append({"topic": prevType, "token": snippet, "startIndex":prevToks[0]["startIndex"] ,"endIndex":prevToks[-1]["endIndex"]})
+    return newtoks, "", []
+
+def replaceMeaninglessPunc(token):
+    return token.replace("(", " ").replace(")", " ").replace(",", " ").strip()
